# Provides:
#   cpuarch
#   num_cpus
#   cpu_model
#   cpu_flags
import re
import shutil


async def load_cpuinfo(hub):
    """
    Return CPU information for AIX systems
    """
    clean = lambda x: x.strip().replace("'", "")

    prtconf = shutil.which("prtconf")
    if prtconf:
        ret = await hub.exec.cmd.run(prtconf)

        # Load cpuarch
        match = re.search(r"(?im)^\s*Processor\s+Type:\s+(\S+)", ret.stdout)
        if match:
            hub.grains.GRAINS.osarch = hub.grains.GRAINS.cpuarch = clean(match.group(1))

        # Load cpu_model
        match = re.search(
            r"(?im)^\s*Processor\s+Implementation\s+Mode:\s+(.*)", ret.stdout
        )
        if match:
            hub.grains.GRAINS.cpu_model = clean(match.group(1))

        # Load num_cpus
        match = re.search(r"(?im)^\s*Number\s+Of\s+Processors:\s+(\S+)", ret.stdout)
        if match:
            hub.grains.GRAINS.num_cpus = int(clean(match.group(1)))


async def load_cpu_flags(hub):
    cpu_flags = []

    lsattr = shutil.which("lsattr")
    if lsattr:
        ret = await hub.exec.cmd.run([lsattr, "-El", "sys0"])
        for line in ret.stdout.strip().splitlines():
            vals = line.split()
            if vals[-1] == "True":
                cpu_flags.append(vals[0].split()[0])

    hub.grains.GRAINS.cpu_flags = sorted(cpu_flags)


async def load_hardware_virtualization(hub):
    hub.grains.GRAINS.hardware_virtualization = False

    #  Specifies whether the machine hardware is MP-capable
    #  (capable of running the multi-processor kernel and supporting more than one processor).
    bootinfo = shutil.which("bootinfo")
    if bootinfo:
        ret = await hub.exec.cmd.run([bootinfo, "-z"])
        hub.grains.GRAINS.hardware_virtualization = ret.stdout.startswith("1")



async def load_defaults(hub):
    # Hard coded grains for AIX systems
    hub.grains.GRAINS.os_family = hub.grains.GRAINS.os = "AIX"
    hub.grains.GRAINS.osmanufacturer = "International Business Machines Corporation"
    hub.grains.GRAINS.virtual = "physical"
    hub.grains.GRAINS.ps = "/usr/bin/ps auxww"



import shutil


async def load_disks(hub):
    """
    Return list of disk devices and work out if they are SSD or HDD.
    """
    SSDs = []
    disks = []

    lsdev = shutil.which("lsdev")
    if lsdev:
        ret = await hub.exec.cmd.run([lsdev, "-c", "disk"])
        for line in ret.stdout.strip().splitlines():
            name, available, disk_type = line.split(maxsplit=2)
            if available == "Available":
                disks.append(name)

    if SSDs:
        # TODO detect which disks are SSDs
        hub.grains.GRAINS.SSDs = sorted(SSDs)
    if disks:
        hub.grains.GRAINS.disks = sorted(disks)



import shutil


async def load_iqn(hub):
    """
    Return iSCSI IQN from an AIX host.
    """
    iscsi_iqn = []
    lsattr = shutil.which("lsattr")

    if lsattr:
        ret = await hub.exec.cmd.run([lsattr, "-E", "-l", "iscsi0"])
        for line in ret.stdout.splitlines():
            line = line.strip()
            if "initiator_name" in line and line[0].isalpha():
                iqns = line.split()
                if len(iqns) > 1:
                    iscsi_iqn.append(iqns[1].rstrip())
    if iscsi_iqn:
        hub.grains.GRAINS.iscsi_iqn = iscsi_iqn



import re
import shutil


async def load_meminfo(hub):
    """
    Return the memory information for AIX systems
    """
    hub.grains.GRAINS.mem_total = 0

    prtconf = shutil.which("prtconf")
    if prtconf:
        ret = await hub.exec.cmd.run([prtconf, "-m"])
        hub.grains.GRAINS.mem_total = int(ret.stdout.strip().split()[2])
    else:
        hub.log.error("Could not find `prtconf` binary in $PATH")


async def load_swapinfo(hub):
    hub.grains.GRAINS.swap_total = 0

    swap = shutil.which("swap")
    if swap:
        # allocated = 688128 blocks used = 2664 blocks free = 685464 blocks
        ret = await hub.exec.cmd.run([swap, "-s"])
        match = re.search(
            "blocks\s+used\s+=\s+(\d+)\s+blocks\s+free\s+=\s+(\d+)\s+blocks",
            ret.stdout.strip(),
        )
        if match:
            hub.grains.GRAINS.swap_total = (
                int(match.group(1)) + int(match.group(2))
            ) * 4
    else:
        hub.log.error("Could not find `swap` binary in $PATH")



import shutil


async def load_techlevel(hub):
    oslevel = shutil.which("oslevel")
    if oslevel:
        hub.grains.GRAINS.osrelease_techlevel = (
            await hub.exec.cmd.run([oslevel, "-r"])
        ).stdout.strip()


async def load_os(hub):
    release_info = lambda osrelease: tuple(
        int(x) if x.strip().isdigit() else x for x in osrelease.split(".")
    )

    oslevel = shutil.which("oslevel")
    uname = shutil.which("uname")
    if oslevel:
        hub.grains.GRAINS.osrelease = (await hub.exec.cmd.run(oslevel)).stdout.strip()
        hub.grains.GRAINS.osrelease_info = release_info(hub.grains.GRAINS.osrelease)
        hub.grains.GRAINS.osmajorrelease = hub.grains.GRAINS.osrelease_info[0]
        if uname:
            hub.grains.GRAINS.osfullname = (
                await hub.exec.cmd.run(uname)
            ).stdout.strip()
            hub.grains.GRAINS.osfinger = (
                f"{hub.grains.GRAINS.osfullname}-{hub.grains.GRAINS.osmajorrelease}"
            )
    hub.grains.GRAINS.oscodename = "unknown"


async def load_build(hub):
    hub.grains.GRAINS.osbuild = "unknown"
    bootinfo = shutil.which("bootinfo")
    if bootinfo:
        ret = await hub.exec.cmd.run([bootinfo, "-m"])
        hub.grains.GRAINS.osbuild = ret.stdout.strip()



"""
Provides
    biosversion
    productname
    manufacturer
    serialnumber
    biosreleasedate
    uuid
"""
import re
import shutil


async def load_lscfg(hub):
    lscfg = shutil.which("lscfg")
    if lscfg:
        ret = await hub.exec.cmd.run([lscfg, "-vpl", "sysplanar0"])

        match = re.search("Microcode Level\.+(\S+)", ret.stdout)
        if match:
            hub.grains.GRAINS.biosversion = match.group(1)

        match = re.search("Microcode Build Date\.+(\S+)", ret.stdout)
        if match:
            hub.grains.GRAINS.biosreleasedate = match.group(1)

        match = re.search("Name:\s+(\S+)", ret.stdout)
        if match:
            hub.grains.GRAINS.manufacturer = match.group(1).split(",")[0]

        match = re.search("Microcode Image\.+(\S+)", ret.stdout)
        if match:
            hub.grains.GRAINS.productname = match.group(1)

        match = re.search("Machine/Cabinet Serial No\.+(\S+)", ret.stdout)
        if match:
            hub.grains.GRAINS.serialnumber = match.group(1)


async def load_uuid(hub):
    lsattr = shutil.which("lsattr")
    if lsattr:
        ret = await hub.exec.cmd.run([lsattr, "-El", "sys0"])
        match = re.search("os_uuid\s+(\S+)", ret.stdout)
        if match:
            hub.grains.GRAINS.uuid = match.group(1)



import os


def __sub_virtual__(hub):
    uname = os.uname()
    if hasattr(uname, "sysname") and uname.sysname == "AIX":
        return True
    else:
        return False, "idem-aix only runs on AIX systems"



import re
import shutil


async def load_gpudata(hub):
    gpus = []

    lshw = shutil.which("lshw")
    if lshw:
        ret = await hub.exec.cmd.run([lshw, "-class", "display"])
        match = re.findall("product:\s+(.*)\s+vendor:\s+(.*)", ret.stdout)
        for m in match:
            gpus.append({"model": m[0], "vendor": m[1]})

    hub.grains.GRAINS.gpus = gpus
    hub.grains.GRAINS.num_gpus = len(hub.grains.GRAINS.gpus)
